\name{save.table}
\alias{save.table}
\alias{save.delim}
\title{Save dataframe in ASCII format}
\description{
These functions do a subset what write.table does.  They are more
memory efficient and may be much faster for large dataframes.
}

\usage{
save.table(x, file = NULL, append = FALSE, sep = " ", eol = "\n",
           na = "NA", col.names = !append, format="%f")
save.delim(x, file = NULL, ..., sep = "\t", na = "")
}

\arguments{
  \item{x}{the object to be written, either a data frame or list.
    It may contain only
    only real, integer (and factor), character and logical components.  If
    \code{x} is list, the components may have different length, in that
    case the shorter ones are filled with \code{NA}-s.}
  \item{file}{output file name.  If \code{NULL}, the output is written
    to the console.
  }
  \item{append}{logical. If TRUE, the output is appended to the file. If
    FALSE, any existing file of the name is destroyed.}
  \item{sep}{the field separator string. Values within each row of x are
    separated by this string.}
  \item{eol}{the character(s) to print at the end of each line (row).}
  \item{na}{the string to use for missing values in the data.}
  \item{col.names}{logical.  If TRUE, column names are written to the
    file.}
  \item{format}{C format string for formatting the real numbers.  Good
    suggestion are either "\%<width>.<dec places>f" for fixed width
    format or "\%<total width>.<digits after .>e" for exponential output.
    }
  \item{...}{Arguments passed straight through from \code{save.delim} to
    \code{save.table}.}
}
\details{
  Both function mimic \code{\link{write.table}} but are less general.
  They differ only by default settings.  The function does not
  handle variables in vector form but as scalar values and hence the
  formatting
  differs slightly from what \code{write.table} does. (values are
  left-, not rigth-justified).}
\author{Ott Toomet \email{siim@obs.ee}.}
\seealso{\code{\link{write.table}}, \code{\link{write.matrix}}}

\examples{
## Save iris-data as CSV file with Sepal.length > 7 removed
data(iris)
iris$Sepal.Length[iris$Sepal.Length>7] <- NA
save.table(iris, file="Iris.csv", sep=",", na="")
save.table(iris, file="Iris2.csv", sep=",", format="\%4.1f")
}
\keyword{IO}
