#include <R.h>
#include <Rdefines.h>
#include <stdio.h>
#include <string.h>

static char *SaveString(SEXP sxp, int offset);

SEXP do_savetable(SEXP data_frame, 
		  SEXP arg_sep,
		  SEXP arg_eol,
		  SEXP arg_na,
		  SEXP col_names,
		  SEXP arg_format)
{
  SEXP names, levels;
  SEXP variable;
  char *thislev;
  char *sep, *eol, *na, *format;
  int int_value;
  int i, j, len, N, M;
  int warn = 0;

  if(!(isNewList(data_frame))) 
    PROBLEM "<df> must be a dataframe!\n" RECOVER( NULL_ENTRY);
  if(!(isString(arg_sep)))
      PROBLEM "<sep> must be a string!\n" RECOVER(NULL_ENTRY);
  if(!(isString(arg_eol)))
      PROBLEM "<eol> must be a string!\n" RECOVER(NULL_ENTRY);
  if(!(isString(arg_na)))
      PROBLEM "<na> must be a string!\n" RECOVER(NULL_ENTRY);
  if(!(isLogical(col_names)))
      PROBLEM "<col.names> must be a logical!\n" RECOVER(NULL_ENTRY);
  if(!(isString(arg_format)))
      PROBLEM "<format> must be a string!\n" RECOVER(NULL_ENTRY);
  sep = SaveString(arg_sep, 0);
  eol = SaveString(arg_eol, 0);
  na = SaveString(arg_na, 0);
  format = SaveString(arg_format, 0);
  names = getAttrib(data_frame, R_NamesSymbol);
  M = length(data_frame);
  /* Now find the component with maximum length.  Normally, all the
     components should have the same length, as in the case of data frame */
  N = 0;
  for(i = 0; i < M; i++) {
      len = length( VECTOR_ELT(data_frame, i));
      if(len > N)
	  N = len;
  }
  /* Now print the dataframe */
  /* first the names, if requested */
  if(LOGICAL(col_names)[0]) {
      for( j = 0; j < M; j++) {
	  Rprintf("%s", CHAR( STRING_ELT( names, j)));
	  if( j < M - 1) {
	      Rprintf("%s", sep);
	  }
      }
      Rprintf( "%s", eol);
  }
  /* And now the data */
  for( i = 0; i < N; i++) {
      /* go through all the rows */
    for( j = 0; j < M; j++) {
	/* through all the columns */
	/* Check, if the component is long enough.  If not, print NA */
	if(i >= length(VECTOR_ELT(data_frame, j))) {
	      Rprintf( "%s", na);
	}
	else {
	    /* The component was long enough, check and print it */
	    variable = VECTOR_ELT(data_frame, j);
	    switch( TYPEOF( variable)) {
		/* process the variables according to their type.  Only
		   following types are allowed: real, string, integer (factor)
		   and logical.  For every type, possible NA value is checked */
	    case REALSXP:
		if(ISNAN(REAL(variable)[i])) {
		    Rprintf( "%s", na);
		    break;
		}
		Rprintf(format, REAL(variable)[i]);
		break;
	    case STRSXP:
		if(STRING_ELT(variable, i) == NA_STRING) {
		    Rprintf( "%s", na);
		    break;
		}
		Rprintf( "%s", CHAR(STRING_ELT(variable, i)));
		break;
	    case INTSXP:
		int_value = INTEGER(variable)[i];
		if(int_value == NA_INTEGER) {
		    Rprintf( "%s", na);
		    break;
		}
		if( isFactor( variable)) {
		    levels = GET_LEVELS(variable);
		    thislev = CHAR(STRING_ELT(levels, int_value - 1));
		    /* NB! first level has value 1, not 0 */
		    Rprintf( "%s", thislev);
		}
		else {
		    Rprintf( "%s", CHAR(StringFromInteger(int_value, &warn)));
		}
		break;
	    case LGLSXP:
		if(LOGICAL(variable)[i] == NA_LOGICAL) {
		    Rprintf( "%s", na);
		    break;
		}
		Rprintf( "%s", 
			CHAR(StringFromLogical(LOGICAL(variable)[i], &warn)));
		break;
	    default:
		PROBLEM "Data frame must contain only\n"
		    "reals, strings, logicals and integers (or factors)!\n" 
		    RECOVER(NULL_ENTRY);
	    }
	} /* The component/variable was long enough to be printed */
	if( j < M - 1) 
	    Rprintf( "%s", sep);
    } /* go through all columns */
    Rprintf( eol);
  } /* go through all rows */
  return( R_NilValue);
}


static char *SaveString(SEXP sxp, int offset)
/* taken from devices.c, removed type check */
{
    char *s;
    s = R_alloc(strlen(CHAR(STRING_ELT(sxp, offset)))+1, sizeof(char));
    strcpy(s, CHAR(STRING_ELT(sxp, offset)));
    return s;
}
